# -*- coding: utf8 -*-
"""Abstract base class for backends"""
from abc import ABCMeta


class BackendInterface(object):
    """
    Backend interface, subclass to implement a concrete backend.


    All methods without a method body need to be implemented in a backend.
    """
    __metaclass__ = ABCMeta

    def get_map(self, dist=None, group=None, name=None):
        """
        get a map of entry points, filtered by

        :param dist: distribution name or sequence of distribution names
        :param groups: single group name or sequence of group names
        :param name: entry point name pattern or sequence of name patterns

        The map is structured as follows::

            map = {
                    '<group>': {
                        '<entrypoint name>: <EntryPoint instance>
                        ...
                    },
                    ...
                },
        """

    def iter_group(self, group):
        """
        returns a list of entry points for the given group name
        """

    def get_group_names(self):
        """
        returns a list of group names
        """

    def get_dist_names(self):
        """
        returns a list of distribution names
        """

    def get_dist_map(self, dist):
        """
        returns a map {group:[entry_points, ...], ...} for the given dist name
        """

    def write_pr_dist(self, dist):
        """
        add a distribution, empty by default
        """

    def write_st_dist(self, dist):
        """
        add a distribution during it's installation
        """

    def write_dist(self, distname, entry_point_map=None):
        """
        take a distribution's project name, add the distribution
        """
        if entry_point_map:
            self.write_dist_map(distname=distname, entry_point_map=entry_point_map)
        else:
            dist = self.pr_dist_from_name(distname)
            self.write_pr_dist(dist)

    def write_dist_map(self, distname, entry_point_map=None):
        """Write a distribution given the name and entry point map"""

    def rm_dist(self, distname):
        """
        removes a distribution completely
        """

    def clear(self):
        """Clears all stored entry points"""

    @staticmethod
    def pr_dist_map(dist):
        dname = dist.project_name
        epmap = dist.get_entry_map()
        return dname, epmap

    @staticmethod
    def pr_dist_from_name(distname):
        from pkg_resources import get_distribution
        dist = get_distribution(distname)
        return dist



# -*- coding: utf8 -*-
"""A backend that uses a json file to store entry points."""
import json

import six

from reentry.abcbackend import BackendInterface
from reentry.entrypoint import EntryPoint


class JsonBackend(BackendInterface):
    """
    Backend using json
    """

    def __init__(self, datafile=None):
        super(JsonBackend, self).__init__()
        from os.path import join, dirname, exists
        self.datafile = join(dirname(__file__), 'js_data')
        self.datafile = datafile or self.datafile
        if not exists(self.datafile):
            with open(self.datafile, 'w') as datafp:
                datafp.write('{}')
        self.epmap = self.read()

    def read(self):
        """
        read state from storage
        """
        with open(self.datafile, 'r') as cache_file_obj:
            return json.load(cache_file_obj)

    def write(self):
        """
        write the current state to storage
        """
        with open(self.datafile, 'w') as cache_file_obj:
            json.dump(self.epmap, cache_file_obj)

    def write_pr_dist(self, dist):
        """
        add a distribution, empty by default
        """
        dname = dist.project_name
        epmap = dist.get_entry_map()
        self.write_dist_map(dname, epmap)
        return dname, epmap

    def write_dist_map(self, distname, entry_point_map=None):
        dname = distname
        # extract entry points that are clearly not for plugins
        entry_point_map.pop('console_scripts', None)
        entry_point_map.pop('gui_scripts', None)
        entry_point_map.pop('distutils.commands', None)
        entry_point_map.pop('distutils.setup_keywords', None)
        entry_point_map.pop('setuptools.installation', None)
        entry_point_map.pop('setuptools.file_finders', None)
        entry_point_map.pop('egg_info.writers', None)
        entry_point_map = {k: {kk: str(vv) for kk, vv in six.iteritems(v)} for k, v in six.iteritems(entry_point_map)}
        # update entry point storage
        # --> only if there is something to update though
        if entry_point_map:
            if not self.epmap.get(dname):
                self.epmap[dname] = {}
            self.epmap[dname].update(entry_point_map)
            self.write()

    def write_st_dist(self, dist):
        """Add a distribution by name."""
        return self.write_pr_dist(self.pr_dist_from_name(dist))

    def write_install_dist(self, dist):
        """Add a distribution during it's install."""
        distname = dist.get_name()
        entrypoint_map = {}
        dist_map = {}
        if hasattr(dist, 'entry_points'):
            dist_map = dist.entry_points or {}
        for group, entrypoint_list in dist_map.items():
            entrypoint_map[group] = {}
            for entrypoint_string in entrypoint_list:
                entry_point = EntryPoint.parse(entrypoint_string)
                entrypoint_map[group][entry_point.name] = entrypoint_string
        self.write_dist_map(distname, entrypoint_map)
        return distname, entrypoint_map

    def iter_group(self, group):
        """Iterate over entry points within a given group."""
        for dist in self.epmap:
            for _, entry_point_spec in six.iteritems(self.epmap[dist].get(group, {})):
                yield EntryPoint.parse(entry_point_spec)

    def get_pr_dist_map(self, dist):
        return self.get_dist_map(dist.project_name)

    def get_dist_map(self, dist):
        """Return the entry map of a given distribution."""
        dmap = self.epmap.get(dist, {}).copy()
        for gname in dmap:
            for epname in dmap[gname]:
                dmap[gname][epname] = EntryPoint.parse(dmap[gname][epname])
        return dmap

    def get_ep(self, group, name, dist=None):
        """
        Get an entry point.

        :param group: the group name
        :param name: the entry point name
        :param dist: if not given, search in all dists

        if no dist was given, and the search turned up more than one
        entry point with the same name, returns a list of entrypoints
        else, returns an entry point
        """
        if not dist:
            specs = []
            for dist_name in self.epmap.keys():
                spc = self.get_ep(group, name, dist=dist_name)
                if spc:
                    specs.append(spc)
            if len(specs) > 1:
                return specs
            elif len(specs) == 1:
                return specs[0]
        else:
            distribution_map = self.epmap.get(dist, {})
            group_map = distribution_map.get(group, {})
            spec = group_map.get(name)
            if spec:
                return EntryPoint.parse(spec)
        return None

    def get_dist_names(self):
        """
        Returns a list of distribution names
        """
        return self.epmap.keys()

    def get_group_names(self):
        """
        Returns a list of group names
        """
        glist = []
        for dist in self.get_dist_names():
            glist.extend(self.epmap[dist].keys())

        return list(set(glist))

    def rm_dist(self, distname):
        """
        removes a distributions entry points from the storage
        """
        if distname in self.get_dist_names():
            self.epmap.pop(distname)
        self.write()

    def rm_group(self, group):
        """
        removes a group from all dists
        """
        for dist in self.epmap:
            self.epmap[dist].pop(group, None)
        self.write()

    def clear(self):
        """
        completely clear entry_point storage
        """
        self.epmap = {}
        self.write()

    def get_map(self, dist=None, group=None, name=None):
        """See BackendInterface docs."""
        # sanitize dist kwarg
        dist_list = self._dist_list_from_arg(dist)

        # sanitize groups kwarg
        group_list = self._group_list_from_arg(group)

        # sanitize name kwarg
        name_list = _listify(name)

        filtered_entry_points = self._filter_entry_points(dist_list, group_list, name_list)
        entry_point_map = {}
        for entry_point, ep_info in six.iteritems(filtered_entry_points):
            if not ep_info['group'] in entry_point_map:
                entry_point_map[ep_info['group']] = {}
            entry_point_map[ep_info['group']][ep_info['name']] = EntryPoint.parse(entry_point)

        return entry_point_map

    def _filter_groups_by_distribution(self, distribution_list, group_list=None):
        """List only groups (optionally from a given list of groups) registered for the given list of distributions"""
        if group_list is None:
            group_list = self.get_group_names()
        group_set = set()
        for distribution in distribution_list:
            if distribution not in self.epmap:
                raise ValueError("The {} distribution was not found.".format(distribution))
            else:
                group_set.update([group_name for group_name in self.epmap[distribution].keys() if group_name in group_list])
        return group_set

    def _filter_entry_points(self, dist_list, group_list, name_list):
        """
        Get a flat dict of annotated entry points, filtered by various criteria

        The dict is formatted like _flat_entry_points() output

        filter by::

            * dist_list: list of distribution names
            * group_list: list of group names
            * name_list: list of regex patterns for entry point names

        Example::

            >> backend.epmap

            {
                'A': {
                    'B': {'entry_point_C': 'entry_point_c = A.foo:bar'},
                    ...
                },
                'other_dist': {
                    'B': { ... },
                    ...
                },
                ...
            }

            >> backend._filter_entry_points(dist_list=['A'], group_list=['B'], name_list=['.*C.*'])

            {'B':
                {'entry_point_C': 'entry_point_c = A.foo:bar'}
            }

        """
        entry_points = self._flat_entry_points()

        def matches(entry_point):
            """True if the entry point matches the filters."""
            result = self._match_pattern_list_exact(entry_point['dist'], dist_list)
            result &= self._match_pattern_list_exact(entry_point['group'], group_list)
            result &= self._match_pattern_list_regex(entry_point['name'], name_list)
            return result

        return {k: v for k, v in six.iteritems(entry_points) if matches(v)}

    @staticmethod
    def _match_pattern_list_regex(name, pattern_list):
        """True if the entry point name matches one of a list of regex patterns."""
        import re
        if not pattern_list:
            return True
        return any([re.match(pattern, name) for pattern in pattern_list])

    @staticmethod
    def _match_pattern_list_exact(name, pattern_list):
        if not pattern_list:
            return True
        return bool(name in pattern_list)

    def _group_list_from_arg(self, group_arg):
        group_list = _listify(group_arg)
        if group_list is None:
            group_list = self.get_group_names()
        return group_list

    def _dist_list_from_arg(self, dist_arg):
        dist_list = _listify(dist_arg)
        if dist_list is None:
            dist_list = self.get_dist_names()
        return dist_list

    def _flat_entry_points(self):
        """Get a flat dict of entry points (keys) annotated with {name: .., group: .., dist: ..} (values)"""
        epflat = {}
        for distribution, dist_dict in six.iteritems(self.epmap):
            for group, group_dict in six.iteritems(dist_dict):
                for ep_name, entry_point in six.iteritems(group_dict):
                    epflat[entry_point] = {'name': ep_name, 'group': group, 'dist': distribution}
        return epflat


def _listify(sequence_or_name):
    """Wrap a single name in a list, leave sequences and None unchanged"""
    from collections import Sequence
    if sequence_or_name is None:
        return None
    elif not isinstance(sequence_or_name, Sequence) or isinstance(sequence_or_name, six.string_types):
        return [sequence_or_name]
    return sequence_or_name



# pylint: disable=unused-import,redefined-outer-name
"""Unit test fixtures"""
from os.path import join, dirname

import pytest

TEST_DATA_FILE = join(dirname(__file__), 'test_data.json')


@pytest.fixture
def test_data():
    """yield a temporary test cache file"""
    import tempfile
    import os
    test_data = None
    with open(TEST_DATA_FILE, 'r') as test_data_file_obj:
        test_data = test_data_file_obj.read()

    file_handle, test_file = tempfile.mkstemp()
    with open(test_file, 'w') as test_file_obj:
        test_file_obj.write(test_data)

    yield test_file

    os.close(file_handle)
    os.remove(test_file)


@pytest.fixture
def bkend(test_data):
    """create a backend with test data"""
    from reentry.jsonbackend import JsonBackend
    test_file = test_data
    return JsonBackend(datafile=test_file)


@pytest.fixture
def manager(bkend):
    from reentry.manager import PluginManager
    manager = PluginManager(backend=bkend)
    yield manager



# pylint: disable=unused-import,redefined-outer-name
"""Unit tests for config tools."""
import pytest  # pylint: disable=unused-import
from py import path as py_path  # pylint: disable=no-name-in-module
import six
from six.moves import configparser

from reentry import config


def test_find_config():
    config_file = config.find_config()
    assert isinstance(config_file, py_path.local)


def test_make_config_parser():
    """Make sure we get the right configparser type."""
    parser = config.make_config_parser()
    if six.PY2:
        assert isinstance(parser, configparser.SafeConfigParser)
    else:
        assert isinstance(parser, configparser.ConfigParser)


def test_get_config():
    """Make sure the configparser gets created correctly."""
    parser = config.get_config()

    if six.PY2:
        assert isinstance(parser, configparser.SafeConfigParser)
    else:
        assert isinstance(parser, configparser.ConfigParser)

    assert parser.get('general', 'datadir')


def test_get_datafile():
    datafile = py_path.local(config.get_datafile())

    assert datafile.isfile()
    assert datafile.exists()



# -*- coding: utf8 -*-
"""Light weight entry point implementation"""
import re


class EntryPoint(object):
    """
    Lightweight analogue for pkg_resources.EntryPoint
    """
    pattern = re.compile(r'\s*(?P<name>.+?)\s*=\s*(?P<module>[\w.]+)\s*(:\s*(?P<attr>[\w.]+))?\s*(?P<extras>\[.*\])?\s*$')

    def __init__(self, name, module_name, attrs=(), distname=None):
        self.name = name
        self.module_name = module_name
        self.attrs = attrs
        self.distname = distname

    @classmethod
    def parse(cls, src, distname=None):
        """
        pasted from pkg_resources, fall back on their EntryPoints when extras are required
        """
        match = cls.pattern.match(src)
        res = match.groupdict()
        if res['extras']:
            import pkg_resources as pr
            dist = pr.get_distribution(distname) if distname else None
            return pr.EntryPoint.parse(src, dist=dist)
        attrs = res['attr'].split('.') if res['attr'] else ()
        return cls(res['name'], res['module'], attrs, distname)

    def load(self):
        """
        pasted from pkg_resources
        """
        import functools
        from importlib import import_module
        module = import_module(self.module_name)
        try:
            return functools.reduce(getattr, self.attrs, module)
        except AttributeError as exc:
            raise ImportError(str(exc))

    def __str__(self):
        string_form = '{} = {}'.format(self.name, self.module_name)
        if self.attrs:
            string_form += ':{}'.format('.'.join(self.attrs))
        return string_form

    def __repr__(self):
        return 'reentry.EntryPoint.parse({})'.format(str(self))



"""Find and read user settings."""
import os
import sys

import six
from six.moves import configparser
from py import path as py_path  # pylint: disable=no-name-in-module

__all__ = ['find_config', 'get_config', 'get_datafile']


def find_config():
    """
    Search for a config file in the following places and order:

        * <HOME>/.reentryrc
        * <HOME>/.config/reentry/config
    """
    home = py_path.local(os.path.expanduser('~'))
    rc_file = home.join('.reentryrc')
    config_file = home.join('.config', 'reentry', 'config')
    if home.exists():
        return rc_file
    elif config_file.exists():
        return config_file
    return rc_file


def make_config_parser(*args, **kwargs):
    """Get the correct ConfigParser class depending on python version."""
    if six.PY2:
        return configparser.SafeConfigParser(*args, **kwargs)
    elif six.PY3:
        return configparser.ConfigParser(*args, **kwargs)
    return None


def get_config(config_file_name=find_config().strpath):
    """Create config parser with defaults and read in the config file."""
    default_config_dir = py_path.local(os.path.expanduser('~/.config/reentry'))
    parser = make_config_parser({'datadir': default_config_dir.join('data').strpath})
    parser.add_section('general')
    parser.read([config_file_name])
    return parser


def get_datafile():
    """Create the path to the data file used to store entry points."""
    config = get_config()
    sep = os.path.sep
    pkg_path_filename = py_path.local(sys.executable).strpath.lstrip(sep).replace(sep, '.')
    datafile = py_path.local(config.get('general', 'datadir')).join(pkg_path_filename)
    datafile.ensure()
    if not datafile.read():
        datafile.write('{}')
    return datafile.strpath



"""Command line interface for reentry"""
import sys

import click


@click.group()
def reentry():
    """manage your reentry python entry point cache"""


@reentry.command()
@click.argument('groups', nargs=-1, metavar='PATTERN')
@click.option('-r', '--regex', is_flag=True, help='Treat PATTERNs as regular expresions')
def scan(groups, regex):
    """
    Scan for python entry points to cache for faster loading.

    Scan only for specific PATTERNs or leave empty to scan all
    """
    from reentry import manager

    if regex:
        if not groups:
            # nothing to do
            sys.exit(0)
        import re
        matchstr = re.compile("|".join(['({})'.format(i) for i in groups]))
        manager.scan(group_re=matchstr)
    else:
        manager.scan(groups)


@reentry.command('map')
@click.option('--dist', help='limit map to a distribution')
@click.option('--group', help='limit map to an entry point group')
@click.option('--name', help='limit map to entrypoints that match NAME')
def map_(dist, group, name):
    """Print out a map of cached entry points"""
    import pprint
    from reentry import manager
    if dist is None:
        res = {d: manager.get_entry_map(d, group, name) for d in manager.distribution_names}
    else:
        res = manager.get_entry_map(dist, group, name)
    click.echo(pprint.pformat(res))


@reentry.command('datafile')
def datafile():
    """Print the path to the current datafile."""
    from reentry.config import get_datafile
    click.echo(get_datafile())


@reentry.group('dev')
def dev():
    """Development related commands."""


def echo_call(cmd):
    click.echo('calling: {}'.format(' '.join(cmd)), err=True)


@dev.command()
def coveralls():
    """Run coveralls only on travis."""
    import os
    import subprocess
    if os.getenv('TRAVIS'):
        cmd = ['coveralls']
        echo_call(cmd)
        subprocess.call(cmd)



# -*- coding: utf8 -*-
"""
Define a setuptools extension.


Host usage::

    setup(
        ...,
        reentry_scan=['group1', 'group2'],
        ...
    )

Plugin usage::

    setup(
        ...,
        reentry_register = True,
        ...
    )
"""
from __future__ import print_function
import sys

from reentry.config import get_datafile


def is_bool(value):
    """Return True if `value` is a boolean."""
    return bool(value) == value


def register_dist(dist, attr, value):
    """If value is True, register the distribution's entry points in reentrys storage."""
    from distutils.errors import DistutilsSetupError  # pylint: disable=import-error,no-name-in-module
    # assert is boolean
    if not is_bool(value):
        raise DistutilsSetupError('{} must be a boolean, got {}'.format(attr, value))

    if value:
        print('registering entry points with reentry...', file=sys.stderr)
        from reentry import manager
        dist_name, entry_point_map = manager.register(dist)
        print('... registered to {}'.format(get_datafile()), file=sys.stderr)
        print('Following entrypoints were registered\n', file=sys.stderr)
        print(manager.format_map({dist_name: entry_point_map}), file=sys.stderr)


def ensure_list(value, attr):
    """raise an error if `value` is not a list"""
    from distutils.errors import DistutilsSetupError  # pylint: disable=import-error,no-name-in-module
    if not isinstance(value, list):
        raise DistutilsSetupError('{} must be a list, got {}'.format(attr, value.__class__))


def scan_for_installed(dist, attr, value):  # pylint: disable=unused-argument
    """
    scan for entry points of the given groups in the already installed
    distributions

    :param value: a list of groups names to scan for
    """
    ensure_list(value, attr)
    if value:
        from reentry import manager
        print('scanning for plugins in groups {}...'.format(value), file=sys.stderr)
        print('Current entry point map at {}:'.format(get_datafile()), file=sys.stderr)
        print(manager.format_map(manager.get_entry_map()), file=sys.stderr)
        scanned_map = manager.scan(groups=value, group_re=False, nodelete=True)
        print('... plugin scanning done.', file=sys.stderr)
        print('Replaced following parts of the map at {}:'.format(get_datafile()), file=sys.stderr)
        print(manager.format_map(scanned_map), file=sys.stderr)
        print('Current entry point map at {}:'.format(get_datafile()), file=sys.stderr)
        print(manager.format_map(manager.get_entry_map()), file=sys.stderr)



# -*- coding: utf8 -*-
"""Manager for entry point based plugins. Main client facing API"""
from __future__ import print_function
import six

from reentry.jsonbackend import JsonBackend

__all__ = ['PluginManager', 'get_default_backend', 'DEFAULT_MANAGER']


def get_default_backend():
    from reentry.config import get_datafile
    return JsonBackend(datafile=get_datafile())


class PluginManager(object):
    """Manage a set of cached entry points"""

    def __init__(self, backend=get_default_backend()):
        self._backend = backend

    def iter_entry_points(self, group, name=None):
        """
        returns all registered entry points in `group`, or if `name` is given,
        only the ones matching `name`.

        The backend may only load pkg_resources if any of the entry points contain extras requirements
        """
        for entry_point in self._backend.iter_group(group):
            if name:
                if name in entry_point.name:
                    yield entry_point
            else:
                yield entry_point

    def get_entry_map(self, dist_names=None, groups=None, ep_names=None):
        """
        return the entry point map for `group` or the whole map for `dist`

        The backend may only load pkg_resources if any of the entry points contain extras requirements
        """
        return self._backend.get_map(dist=dist_names, group=groups, name=ep_names)

    @staticmethod
    def format_map(entry_point_map):
        return '\n'.join(['{} -> {}'.format(dname, dmap) for dname, dmap in entry_point_map.items()])

    def register(self, distribution):
        """
        Registers the distribution's entry points with the backend.

        The backend may load pkg_resources to resolve the distribution name

        Takes either a string or a Distribution object as passed by setuptools to hooks during install.
        """
        if isinstance(distribution, six.string_types):
            dist_name, entry_point_map = self._backend.write_st_dist(distribution)
        else:
            dist_name, entry_point_map = self._backend.write_install_dist(distribution)
        return dist_name, entry_point_map

    def scan(self, groups=None, group_re=None, nocommit=False, nodelete=False):
        """
        walks through all distributions available and registers entry points or only those in `groups`
        """
        import pkg_resources as pr
        pr_env = pr.AvailableDistributions()
        pr_env.scan()
        if not groups and group_re:
            groups = []

        if group_re:
            groups.extend({group for group in self._backend.get_group_names() if group_re.match(group)})

        if not nocommit and nodelete:
            if groups:
                for group in groups:
                    self._backend.rm_group(group)
            else:
                self._backend.clear()

        full_map = {}

        if nodelete:
            full_map = self._backend.ep_map()

        for dists in pr_env._distmap.values():  # pylint: disable=protected-access
            dist = dists[0]
            emap = dist.get_entry_map() or {}
            dname = dist.project_name
            dmap = full_map.get(dname, {})
            if groups:
                new_dmap = {k: v for k, v in six.iteritems(emap) if k in groups}
                dmap.update(new_dmap)
            else:
                dmap.update(emap)
            if not nocommit:
                self._backend.write_dist_map(dname, entry_point_map=dmap)
            full_map[dname] = [dmap]

        return full_map

    def unregister(self, distname):
        """
        unregisters the distribution's entry points with the backend

        The backend may load pkg_resources to resolve the distribution name
        """
        self._backend.rm_dist(distname)

    @property
    def distribution_names(self):
        return self._backend.get_dist_names()


DEFAULT_MANAGER = PluginManager()



# pylint: disable=unused-import,redefined-outer-name
"""Unit tests for js-backend"""
import json

import pytest

from fixtures import test_data, bkend
from reentry.entrypoint import EntryPoint


def test_get_map_default(bkend):
    """Test the backend's get_map method without arguments"""
    ep_map = bkend.get_map()

    assert 'groupB' in ep_map
    assert 'distA.epB' in ep_map['groupB']
    assert 'distB.epB' in ep_map['groupB']
    assert isinstance(ep_map['groupB']['distB.epB'], EntryPoint)


def test_get_map_by_dist(bkend):
    """Test get_map output filtering with dist"""
    ep_map = bkend.get_map(dist='distA')
    assert 'distA.epB' in ep_map['groupB']
    assert 'distB.epB' not in ep_map['groupB']

    ep_map = bkend.get_map(dist=['distA', 'distB'])
    assert 'distA.epB' in ep_map['groupB']
    assert 'distB.epB' in ep_map['groupB']


def test_get_map_by_group(bkend):
    """Test get_map output filtering with group"""
    ep_map = bkend.get_map(group='groupA')
    assert 'groupA' in ep_map
    assert 'groupB' not in ep_map

    ep_map = bkend.get_map(group=['groupA', 'groupB'])
    assert 'groupA' in ep_map
    assert 'groupB' in ep_map
    assert 'groupC' not in ep_map


def test_get_map_by_name(bkend):
    """Test get_map ouptut filering with name"""
    ep_map = bkend.get_map(name=r'.*\.epB')
    assert 'distA.epB' in ep_map['groupB']
    assert 'distB.epB' in ep_map['groupB']
    assert 'groupA' not in ep_map

    ep_map = bkend.get_map(name=[r'.*\.epB', 'distB.epC'])
    assert 'distA.epB' in ep_map['groupB']
    assert 'distB.epC' in ep_map['groupC']
    assert 'groupA' not in ep_map


def test_iter_group(bkend):
    ep_list = list(bkend.iter_group('groupB'))
    assert 'distA.epB' in [i.name for i in ep_list]
    assert 'distB.epB' in [i.name for i in ep_list]


def test_group_names(bkend):
    group_names = bkend.get_group_names()
    assert 'groupA' in group_names
    assert 'groupB' in group_names
    assert 'groupC' in group_names


def test_dist_names(bkend):
    dist_names = bkend.get_dist_names()
    assert 'distA' in dist_names
    assert 'distB' in dist_names


def test_write_dist_map(bkend):
    entry_point_map = {'test_group': {'test_ep': 'test_ep = test_dist.test_module:test_member'}}
    bkend.write_dist_map(distname='test_dist', entry_point_map=entry_point_map)
    assert 'test_dist' in list(bkend.get_dist_names())
    assert bkend.get_map(dist='test_dist')


def test_write_pr_dist(bkend):
    """Test caching entry points for a given pkg_resources - distribution"""
    this_dist = bkend.pr_dist_from_name('reentry')
    bkend.write_pr_dist(this_dist)
    assert 'reentry' in list(bkend.get_dist_names())
    assert bkend.get_map(dist='reentry')


def test_write_st_dist(bkend):
    """Test caching entry points for a distribution given by name."""
    bkend.write_st_dist('reentry')
    assert 'reentry' in list(bkend.get_dist_names())
    assert bkend.get_map(dist='reentry')


def test_write_install_dist(bkend):
    """Test caching entry points for a pkg_resources - distribution at install time."""
    this_dist = bkend.pr_dist_from_name('reentry')
    this_dist.get_name = lambda: 'reentry'
    this_dist.entry_points = {'foo': ['bar = foo.bar:baz']}
    bkend.write_install_dist(this_dist)
    assert 'reentry' in list(bkend.get_dist_names())
    assert 'foo' in bkend.get_map(dist='reentry')


def test_rm_dist(bkend):
    bkend.rm_dist('distA')
    assert 'distA' not in bkend.epmap


def test_clear(bkend):
    bkend.clear()
    assert not bkend.get_map()



# pylint: disable=unused-import,redefined-outer-name
"""Unit tests for manager functions"""
import pytest

from fixtures import test_data, bkend, manager


def test_get_entry_map(manager):
    """test full map"""
    epmap = manager.get_entry_map()
    assert 'groupA' in epmap
    assert 'groupB' in epmap
    assert 'groupC' in epmap


def test_get_entry_map_dist(manager):
    """
    The map for distA in the fixture should contain
    a group and an entry point
    """
    epmap = manager.get_entry_map(dist_names='distA')
    assert 'groupA' in epmap
    assert 'groupB' in epmap
    assert 'groupC' not in epmap
    assert 'distA.epA' in epmap.get('groupA', {})
    assert 'distA.epB' in epmap.get('groupB', {})


def test_entry_map_dist_group(manager):
    """Check filtering entry point map for a distribution by group"""
    epmap = manager.get_entry_map(dist_names='distA', groups='groupA')
    assert 'groupA' in epmap
    assert 'groupB' not in epmap
    assert 'distA.epA' in epmap.get('groupA', {})


def test_iter_entry_points(manager):
    """Test the drop-in replacement for pkg_resources.iter_entry_points"""
    entry_points = manager.iter_entry_points(group='groupA')
    assert 'distA.epA' in [e.name for e in entry_points]

    entry_points = list(manager.iter_entry_points(group='groupB'))
    assert 'distA.epB' in [e.name for e in entry_points]
    assert 'distB.epB' in [e.name for e in entry_points]


def test_register(manager):
    """Test registering a distribution"""
    manager.register('reentry')
    ep_map = manager.get_entry_map(dist_names='reentry')
    assert 'test_entry_points' in ep_map
    assert 'console_scripts' not in ep_map


def test_scan(manager):
    """Test scanning for entry points"""
    manager.scan()
    ep_map = manager.get_entry_map(dist_names='reentry')
    assert 'test_entry_points' in ep_map
    assert 'console_scripts' not in ep_map


def test_scan_group(manager):
    manager.scan(groups=['test_entry_points'])
    ep_map = manager.get_entry_map(dist_names='reentry')
    assert 'test_entry_points' in ep_map
    assert 'console_scripts' not in ep_map


def test_unregister(manager):
    manager.unregister('distA')
    assert 'distA' not in manager.distribution_names



# -*- coding: utf8 -*-
"""
Install configuration for setuptools / pip
"""
from os import path
from setuptools import setup, find_packages

README_PATH = path.join(path.dirname(path.abspath(__file__)), 'README.rst')
with open(README_PATH, 'r') as readme:
    LONG_DESCRIPTION = readme.read()

VERSION = '1.2.0a5'

setup(
    name='reentry',
    version=VERSION,
    author='Rico Haeuselmann',
    license='MIT License',
    description='A plugin manager based on setuptools entry points mechanism',
    long_description=LONG_DESCRIPTION,
    packages=find_packages(),
    include_package_data=True,
    package_data={'': ['js_data', 'README.rst']},
    entry_points={
        'distutils.setup_keywords': ['reentry_register = reentry.hooks:register_dist', 'reentry_scan = reentry.hooks:scan_for_installed'],
        'console_scripts': ['reentry = reentry.cli:reentry'],
        'test_entry_points': ['test = reentry.cli:reentry']
    },
    classifiers=[
        'License :: OSI Approved :: MIT License', 'Programming Language :: Python :: 2', 'Programming Language :: Python :: 3',
        'Development Status :: 5 - Production/Stable', 'Environment :: Plugins', 'Intended Audience :: Developers',
        'Topic :: Software Development'
    ],
    install_requires=['setuptools >= 18.5', 'click', 'six', 'py'],
    extras_require={
        'dev': ['pre-commit', 'prospector', 'pylint', 'flake8', 'pytest', 'yapf', 'coverage', 'pytest-cov', 'tox', 'packaging']
    })
